@@ -20,7 +20,8 @@
 #' #This is an internal function, no example provided
 kfunc <- function(dist_mat,start,end,step,Lt,n,w){
   breaks <- seq(start,end,step)
-  t1 <- Lt/(n*(n-1))
+  #t1 <- Lt/(n*(n-1))
+  t1 <- (n-1)/Lt
   k_values <- sapply(breaks,function(dist){
     int_mat <- t(t(dist_mat<=dist) * w)
     diag(int_mat) <- 0
@@ -52,7 +53,8 @@ kfunc <- function(dist_mat,start,end,step,Lt,n,w){
 gfunc <- function(dist_mat,start,end,step,width,Lt,n,w){
   breaks <- seq(start,end,step)
   width <- width/2
-  t1 <- Lt/(n*(n-1))
+  #t1 <- Lt/(n*(n-1))
+  t1 <- (n-1)/Lt
   k_values <- sapply(breaks,function(dist){
     int_mat <- t(t(dist_mat<=dist+width & dist_mat>=dist-width) * w)
     diag(int_mat) <- 0
@@ -89,7 +91,8 @@ gfunc <- function(dist_mat,start,end,step,width,Lt,n,w){
 #' #This is an internal function, no example provided
 cross_kfunc <- function(dist_mat,start,end,step,Lt,na,nb,wa,wb){
   breaks <- seq(start,end,step)
-  t1 <- Lt/(na*nb)
+  #t1 <- Lt/(na*nb)
+  t1 <- (na/Lt)
 
   # note : in the matrix, the rows are the b points
   # and the columns are the a points
@@ -129,7 +132,8 @@ cross_kfunc <- function(dist_mat,start,end,step,Lt,na,nb,wa,wb){
 cross_gfun <- function(dist_mat,start,end,step,width,Lt,na,nb,wa,wb){
   breaks <- base::seq(from = start,to = end, by = step)
   width <- width/2
-  t1 <- Lt/(na*nb)
+  #t1 <- Lt/(na*nb)
+  t1 <- (na/Lt)
   g_values <- sapply(breaks,function(dist){
     d1 <- dist + width
     d2 <- dist - width
@@ -179,17 +183,17 @@ randomize_distmatrix2 <- function(graph, edge_df, n, resolution, nsim, start_ver
     start_node <- all_names[this_edge$start_oid]
     end_node <- all_names[this_edge$end_oid]
     dists <- rep(resolution, floor(this_edge$weight/resolution))
+    if(sum(dists) == this_edge$weight){
+      dists <- dists[1:(length(dists)-1)]
+    }
     names(dists) <- paste("fict",i,1:length(dists), sep="_")
-    starts <- c(start_node, names(dists))
-    ends <- c(names(dists), end_node)
-    dists <- c(dists, this_edge$weight - sum(dists))
-    df <- data.frame(starts = starts,
-                     ends = ends,
-                     weight = dists)
-    return(df)
+    return(data.frame(starts = c(start_node, names(dists)),
+                      ends = c(names(dists), end_node),
+                      weight = c(dists, this_edge$weight - sum(dists))))
   })
 
   all_elements <- do.call(rbind, vertices_and_distances)
+  all_elements <- subset(all_elements,all_elements$weight>0)
 
   ## step 3 : creating a new graph with the new vertices and edges
   new_graph <- igraph::graph_from_data_frame(all_elements, directed = FALSE)
@@ -215,8 +219,6 @@ randomize_distmatrix2 <- function(graph, edge_df, n, resolution, nsim, start_ver
     new_vert <- sample(verts,size = n, replace = F)
     if (is.null(start_vert)){
       dist_mat <- igraph::distances(tot_graph,v = new_vert, to = new_vert)
-
-
     }else{
       dist_mat <- igraph::distances(tot_graph,v = start_vert, to = new_vert)
 
@@ -245,7 +247,6 @@ randomize_distmatrix2 <- function(graph, edge_df, n, resolution, nsim, start_ver
 randomize_distmatrix <- function(graph, edge_df, n, start_vert = NULL){
 
   vec_runif <- Vectorize(runif, vectorize.args = c("max"))
-  ## step2 : generate the random scenario
 
   ## prefered case where points are randomly located on edges
 
@@ -255,25 +256,33 @@ randomize_distmatrix <- function(graph, edge_df, n, start_vert = NULL){
                          prob = 1/edge_df$weight * edge_df$probs)
 
   sel_edges <- edge_df[sel_edges_id,]
-
   sel_edges_len <- sel_edges$weight
 
+  # preparing some variables for later
   start_oids <- sel_edges$start_oid
   end_oids <- sel_edges$end_oid
   all_names <- names(igraph::V(graph))
 
+  # finding the start nodes and end nodes of the selected edges
   start_names <- all_names[start_oids]
   end_names <- all_names[end_oids]
 
-  #b. calculating the position of the point on the edge
+  #b. calculating the position of the point on the edges
+  # each edge will receive one point
   dists <- vec_runif(n=1,min = 0, max = sel_edges_len)
+  # creating virtual names for the new vertices
   new_vert <- paste0(rep("virt_"),seq_len(length(dists)))
+
   #c. creating the new edges and nodes as another graph
   df <- data.frame(start = c(start_names,new_vert),
                    end = c(new_vert,end_names),
                    weight = c(dists,(sel_edges_len - dists)))
+
+  #d. and then merging the graphs
   new_graph <- igraph::graph_from_data_frame(df, directed = FALSE)
   tot_graph <- igraph::union(graph,new_graph, byname = TRUE)
+
+  # We just need to merge the weights of the graphs
   ws <- igraph::E(tot_graph)
   df_tmp <- data.frame("w1" = ws$weight_1,
                        "w2" = ws$weight_2)
@@ -304,10 +313,10 @@ randomize_distmatrix <- function(graph, edge_df, n, start_vert = NULL){
 #### execution k functions ####
 # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-#' @title Network k and g functions (experimental)
+#' @title Network k and g functions (maturing)
 #'
 #' @description Calculate the k and g functions for a set of points on a
-#'   network (experimental).
+#'   network (maturing).
 #'
 #' @details The k-function is a method to characterize the dispersion of a set
 #'   of points. For each point, the numbers of other points in subsequent radii
@@ -321,45 +330,23 @@ randomize_distmatrix <- function(graph, edge_df, n, start_vert = NULL){
 #'   can expect from randomness and vice-versa. The function also calculates the
 #'   g-function, a modified version of the k-function using rings instead of
 #'   disks. The width of the ring must be chosen. The main interest is to avoid
-#'   the cumulative effect of the classical k-function.
+#'   the cumulative effect of the classical k-function. This function is maturing,
+#'   it works as expected (unit tests) but will probably be modified in the
+#'   future releases (gain speed, advanced features, etc.).
 #'
-#' @param lines A SpatialLinesDataFrame with the sampling points. The geometries
-#'   must be a SpatialLinesDataFrame (may crash if some geometries are invalid)
-#' @param points A SpatialPointsDataFrame representing the points on the
-#'   network. These points will be snapped on the network.
-#' @param start A double, the start value for evaluating the k and g functions
-#' @param end A double, the last value for evaluating the k and g functions
-#' @param step A double, the jump between two evaluations of the k and g
-#'   function
-#' @param width The width of each donut for the g-function
-#' @param nsim An integer indicating the number of Monte Carlo simulations
-#'   required
-#' @param conf_int A double indicating the width confidence interval (default =
-#'   0.05)
-#' @param digits An integer indicating the number of digits to retain for the
-#'   spatial coordinates
-#' @param tol When adding the points to the network, specify the minimum
-#'   distance between these points and the lines' extremities. When points are
-#'   closer, they are added at the extremity of the lines.
-#' @param resolution When simulating random points on the network, selecting a
-#'   resolution will reduce greatly the calculation time. When resolution is null
-#'   the random points can occur everywhere on the graph. If a value is specified,
-#'   the edges are split according to this value and the random points are
-#'   selected vertices on the new network.
-#' @param agg A double indicating if the events must be aggregated within a
-#'   distance. If NULL, the events are aggregated by rounding the coordinates.
-#' @param verbose A Boolean indicating if progress messages should be displayed.
+#' @template kfunctions-arg
+#' @template common_kfunctions-arg
 #'
-#' @return A list with the following values : \cr \itemize{ \item{plotk}{A
-#'   ggplot2 object representing the values of the k-function} \item{plotg}{A
-#'   ggplot2 object representing the values of the g-function} \item{values}{A
+#' @return A list with the following values : \cr \itemize{ \item{plotk}{ A
+#'   ggplot2 object representing the values of the k-function} \item{plotg}{ A
+#'   ggplot2 object representing the values of the g-function} \item{values}{ A
 #'   DataFrame with the values used to build the plots} }
 #' @importFrom stats quantile
 #' @importFrom rgeos gLength
 #' @importFrom ggplot2 ggplot geom_ribbon geom_path aes_string labs
 #' @export
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' networkgpkg <- system.file("extdata", "networks.gpkg", package = "spNetwork", mustWork = TRUE)
 #' eventsgpkg <- system.file("extdata", "events.gpkg", package = "spNetwork", mustWork = TRUE)
 #' main_network_mtl <- rgdal::readOGR(networkgpkg,layer="main_network_mtl", verbose=FALSE)
@@ -400,14 +387,16 @@ kfunctions <- function(lines, points, start, end, step, width, nsim, conf_int =
     print("Snapping points on lines ...")
   }
   snapped_events <- snapPointsToLines2(points,lines,idField = "oid")
-  new_lines <- add_vertices_lines(lines,snapped_events,
-                                  snapped_events$nearest_line_id, tol)
+  new_lines <- split_lines_at_vertex(lines, snapped_events,
+                                     snapped_events$nearest_line_id, tol)
+  # new_lines <- add_vertices_lines(lines,snapped_events,
+  #                                 snapped_events$nearest_line_id, tol)
 
   ## step3 : splitting the lines
   if (verbose){
     print("Building graph ...")
   }
-  new_lines <- simple_lines(new_lines)
+  #new_lines <- simple_lines(new_lines)
   new_lines$length <- gLength(new_lines,byid = TRUE)
   new_lines <- subset(new_lines,new_lines$length>0)
   new_lines <- remove_loop_lines(new_lines,digits)
@@ -415,9 +404,11 @@ kfunctions <- function(lines, points, start, end, step, width, nsim, conf_int =
   new_lines <- new_lines[c("length","oid","probs")]
   Lt <- gLength(new_lines)
 
+  new_lines$weight <- gLength(new_lines, byid = TRUE)
+
   ## step4 : building the graph for the real case
   graph_result <- build_graph(new_lines,digits = digits,
-                              line_weight = "length",
+                              line_weight = "weight",
                               attrs = TRUE)
   graph <- graph_result$graph
   nodes <- graph_result$spvertices
@@ -529,44 +520,20 @@ kfunctions <- function(lines, points, start, end, step, width, nsim, conf_int =
 }
 
 
-#' @title Network k and g functions (multicore, experimental)
+#' @title Network k and g functions (multicore, maturing)
 #'
 #' @description Calculate the k and g functions for a set of points on a network
 #'   with multicore support. For details, please see the function kfunctions.
-#'   (experimental)
+#'   (maturing)
 #'
 #' @details For details, please look at the function kfunctions.
 #'
-#' @param lines A SpatialLinesDataFrame with the sampling points. The geometries
-#'   must be a SpatialLinesDataFrame (may crash if some geometries are invalid)
-#' @param points A SpatialPointsDataFrame representing the points on the
-#'   network. These points will be snapped on the network.
-#' @param start A double, the start value for evaluating the k and g functions
-#' @param end A double, the last value for evaluating the k and g functions
-#' @param step A double, the jump between two evaluations of the k and g
-#'   functions
-#' @param width The width of each donut for the g-function
-#' @param nsim An integer indicating the number of Monte Carlo simulations
-#'   required
-#' @param conf_int A double indicating the width confidence interval (default =
-#'   0.05)
-#' @param digits An integer indicating the number of digits to retain for the
-#'   spatial coordinates
-#' @param tol When adding the points to the network, specify the minimum
-#'   distance between these points and the lines' extremities. When points are
-#'   closer, they are added at the extremity of the lines.
-#' @param resolution When simulating random points on the network, selecting a
-#'   resolution will reduce greatly the calculation time. When resolution is null
-#'   the random points can occur everywhere on the graph. If a value is specified,
-#'   the edges are split according to this value and the random points are
-#'   selected vertices on the new network.
-#' @param agg A double indicating if the events must be aggregated within a
-#'   distance. If NULL, the events are aggregated by rounding the coordinates.
-#' @param verbose A Boolean indicating if progress messages should be displayed
+#' @template kfunctions-arg
+#' @template common_kfunctions-arg
 #'
-#' @return A list with the following values : \cr \itemize{ \item{plotk}{A
-#'   ggplot2 object representing the values of the k-function} \item{plotg}{A
-#'   ggplot2 object representing the values of the g-function} \item{values}{A
+#' @return A list with the following values : \cr \itemize{ \item{plotk}{ A
+#'   ggplot2 object representing the values of the k-function} \item{plotg}{ A
+#'   ggplot2 object representing the values of the g-function} \item{values}{ A
 #'   DataFrame with the values used to build the plots} }
 #'
 #' @importFrom stats quantile
@@ -576,7 +543,7 @@ kfunctions <- function(lines, points, start, end, step, width, nsim, conf_int =
 #'
 #' @export
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' networkgpkg <- system.file("extdata", "networks.gpkg", package = "spNetwork", mustWork = TRUE)
 #' eventsgpkg <- system.file("extdata", "events.gpkg", package = "spNetwork", mustWork = TRUE)
 #' main_network_mtl <- rgdal::readOGR(networkgpkg,layer="main_network_mtl", verbose=FALSE)
@@ -618,13 +585,15 @@ kfunctions.mc <- function(lines, points, start, end, step, width, nsim, conf_int
     print("Snapping points on lines ...")
   }
   snapped_events <- snapPointsToLines2(points,lines,idField = "oid")
-  new_lines <- add_vertices_lines(lines,snapped_events,snapped_events$nearest_line_id,tol)
+  new_lines <- split_lines_at_vertex(lines, snapped_events,
+                                     snapped_events$nearest_line_id, tol)
+  #new_lines <- add_vertices_lines(lines,snapped_events,snapped_events$nearest_line_id,tol)
 
   ## step3 : splitting the lines
   if(verbose){
     print("Building graph ...")
   }
-  new_lines <- simple_lines(new_lines)
+  #new_lines <- simple_lines(new_lines)
   new_lines$length <- gLength(new_lines,byid = TRUE)
   new_lines <- subset(new_lines,new_lines$length>0)
   new_lines <- remove_loop_lines(new_lines,digits)
@@ -718,11 +687,11 @@ kfunctions.mc <- function(lines, points, start, end, step, width, nsim, conf_int
 
   plot_df <- data.frame(
     "obs_k" = k_vals,
-    "lower_k" = k_stats[1,],
-    "upper_k" = k_stats[2,],
+    "upper_k" = k_stats[1,],
+    "lower_k" = k_stats[2,],
     "obs_g" = g_vals,
-    "lower_g" = g_stats[1,],
-    "upper_g" = g_stats[2,],
+    "upper_g" = g_stats[1,],
+    "lower_g" = g_stats[2,],
     "distances" = seq(start,end,step)
   )
 
@@ -761,10 +730,10 @@ kfunctions.mc <- function(lines, points, start, end, step, width, nsim, conf_int
 # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
-#' @title Network cross k and g functions (experimental)
+#' @title Network cross k and g functions (maturing)
 #'
 #' @description Calculate the cross k and g functions for a set of points on a
-#'   network. (experimental)
+#'   network. (maturing)
 #'
 #' @details The cross k-function is a method to characterize the dispersion of a
 #'   set of points (A) around a second set of points (B). For each point in B,
@@ -782,42 +751,17 @@ kfunctions.mc <- function(lines, points, start, end, step, width, nsim, conf_int
 #'   the ring must be chosen. The main interest is to avoid the cumulative
 #'   effect of the classical k-function. Note that the cross k-function of
 #'   points A around B is not necessarily the same as the cross k-function of
-#'   points B around A.
+#'   points B around A. This function is maturing, it works as expected (unit
+#'   tests) but will probably be modified in the future releases (gain speed,
+#'   advanced features, etc.).
 #'
-#' @param lines A SpatialLinesDataFrame with the sampling points. The geometries
-#'   must be a SpatialLinesDataFrame (may crash if some geometries are invalid)
-#' @param pointsA A SpatialPointsDataFrame representing the points to which the
-#'   distances are calculated.
-#' @param pointsB A SpatialPointsDataFrame representing the points from which
-#'   the distances are calculated.
-#' @param start A double, the start value for evaluating the k and g functions
-#' @param end A double, the last value for evaluating the k and g functions
-#' @param step A double, the jump between two evaluations of the k and g
-#'   function
-#' @param width The width of each donut for the g-function
-#' @param nsim An integer indicating the number of Monte Carlo simulations
-#'   required
-#' @param conf_int A double indicating the width confidence interval (default =
-#'   0.05)
-#' @param digits An integer indicating the number of digits to retain for the
-#'   spatial coordinates
-#' @param tol When adding the points to the network, specify the minimum
-#'   distance between these points and the lines' extremities. When points are
-#'   closer, they are added at the extremity of the lines.
-#' @param resolution When simulating random points on the network, selecting a
-#'   resolution will reduce greatly the calculation time. When resolution is null
-#'   the random points can occur everywhere on the graph. If a value is specified,
-#'   the edges are split according to this value and the random points are
-#'   selected vertices on the new network.
-#' @param agg A double indicating if the events must be aggregated within a
-#'   distance. if NULL, then the events are aggregated by rounding the
-#'   coordinates.
-#' @param verbose A Boolean indicating if progress messages should be displayed
+#' @template kross_kfunctions-arg
+#' @template common_kfunctions-arg
 #'
-#' @return A list with the folowing values : \cr \itemize{ \item{plotk}{A
+#' @return A list with the following values : \cr \itemize{ \item{plotk}{ A
 #'   ggplot2 object representing the values of the cross k-function}
-#'   \item{plotg}{A ggplot2 object representing the values of the cross
-#'   g-function} \item{values}{A DataFrame with the values used to build the
+#'   \item{plotg}{ A ggplot2 object representing the values of the cross
+#'   g-function} \item{values}{ A DataFrame with the values used to build the
 #'   plots} }
 #' @importFrom stats quantile
 #' @importFrom rgeos gLength
@@ -825,7 +769,7 @@ kfunctions.mc <- function(lines, points, start, end, step, width, nsim, conf_int
 #' @importFrom grDevices rgb
 #' @export
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' networkgpkg <- system.file("extdata", "networks.gpkg", package = "spNetwork", mustWork = TRUE)
 #' eventsgpkg <- system.file("extdata", "events.gpkg", package = "spNetwork", mustWork = TRUE)
 #' main_network_mtl <- rgdal::readOGR(networkgpkg,layer="main_network_mtl", verbose=FALSE)
@@ -875,14 +819,16 @@ cross_kfunctions <- function(lines, pointsA, pointsB, start, end, step, width, n
                       pointsB[c("type","goid","weight")])
 
   snapped_events <- snapPointsToLines2(all_events, lines, idField = "oid")
-  new_lines <- add_vertices_lines(lines, snapped_events,
+  new_lines <- split_lines_at_vertex(lines, snapped_events,
                                   snapped_events$nearest_line_id, tol)
+  # new_lines <- add_vertices_lines(lines, snapped_events,
+  #                                 snapped_events$nearest_line_id, tol)
 
   ## step3 : splitting the lines
   if(verbose){
     print("Building graph ...")
   }
-  new_lines <- simple_lines(new_lines)
+  #new_lines <- simple_lines(new_lines)
   new_lines$length <- gLength(new_lines ,byid = TRUE)
   new_lines <- subset(new_lines, new_lines$length>0)
   new_lines <- remove_loop_lines(new_lines, digits)
@@ -903,7 +849,7 @@ cross_kfunctions <- function(lines, pointsA, pointsB, start, end, step, width, n
 
   ## step5 : calculating the distance matrix
   dist_mat <- igraph::distances(graph,v = snappedB$vertex_id,
-                                to = snappedA$vertex_id)
+                                to = snappedA$vertex_id, mode = "out")
   ## step6 : calcualte the kfunction and the g function
   if(verbose){
     print("Calculating k and g functions ...")
@@ -1006,44 +952,18 @@ cross_kfunctions <- function(lines, pointsA, pointsB, start, end, step, width, n
 }
 
 
-#' @title Network cross k and g functions (multicore, experimental)
+#' @title Network cross k and g functions (multicore, maturing)
 #'
 #' @description Calculate the cross k and g functions for a set of points on a
-#'   network with multicore support. (experimental)
+#'   network with multicore support. (maturing)
 #'
-#' @param lines A SpatialLinesDataFrame with the sampling points. The geometries
-#'   must be a SpatialLinesDataFrame (may crash if some geometries are invalid)
-#' @param pointsA A SpatialPointsDataFrame representing the points to which the
-#'   distances are calculated.
-#' @param pointsB A SpatialPointsDataFrame representing the points from which
-#'   the distances are calculated.
-#' @param start A double, the start value for evaluating the k and g functions
-#' @param end A double, the last value for evaluating the k and g functions
-#' @param step A double, the jump between two evaluations of the k and g
-#'   function
-#' @param width The width of each donut for the g-function
-#' @param nsim An integer indicating the number of Monte Carlo simulations
-#'   required
-#' @param conf_int A double indicating the width confidence interval (default =
-#'   0.05)
-#' @param digits An integer indicating the number of digits to retain for the
-#'   spatial coordinates
-#' @param tol When adding the points to the network, specify the minimum
-#'   distance between these points and the lines' extremities. When points are
-#'   closer, they are added at the extremity of the lines.
-#' @param resolution When simulating random points on the network, selecting a
-#'   resolution will reduce greatly the calculation time. When resolution is null
-#'   the random points can occur everywhere on the graph. If a value is specified,
-#'   the edges are split according to this value and the random points are
-#'   selected vertices on the new network.
-#' @param agg A double indicating if the events must be aggregated within a
-#'   distance. If NULL, the events are aggregated by rounding the coordinates.
-#' @param verbose A Boolean indicating if progress messages should be displayed
+#' @template kross_kfunctions-arg
+#' @template common_kfunctions-arg
 #'
-#' @return A list with the following values : \cr \itemize{ \item{plotk}{A
+#' @return A list with the following values : \cr \itemize{ \item{plotk}{ A
 #'   ggplot2 object representing the values of the cross k-function}
-#'   \item{plotg}{A ggplot2 object representing the values of the cross
-#'   g-function} \item{values}{A DataFrame with the values used to build the
+#'   \item{plotg}{ A ggplot2 object representing the values of the cross
+#'   g-function} \item{values}{ A DataFrame with the values used to build the
 #'   plots} }
 #' @importFrom stats quantile
 #' @importFrom rgeos gLength
@@ -1051,7 +971,7 @@ cross_kfunctions <- function(lines, pointsA, pointsB, start, end, step, width, n
 #' @importFrom grDevices rgb
 #' @export
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' networkgpkg <- system.file("extdata", "networks.gpkg", package = "spNetwork", mustWork = TRUE)
 #' eventsgpkg <- system.file("extdata", "events.gpkg", package = "spNetwork", mustWork = TRUE)
 #' main_network_mtl <- rgdal::readOGR(networkgpkg,layer="main_network_mtl", verbose=FALSE)
@@ -1104,14 +1024,16 @@ cross_kfunctions.mc <- function(lines, pointsA, pointsB, start, end, step, width
                       pointsB[c("type","goid","weight")])
 
   snapped_events <- snapPointsToLines2(all_events, lines, idField = "oid")
-  new_lines <- add_vertices_lines(lines, snapped_events,
+  new_lines <- split_lines_at_vertex(lines, snapped_events,
                                   snapped_events$nearest_line_id, tol)
+  # new_lines <- add_vertices_lines(lines, snapped_events,
+  #                                 snapped_events$nearest_line_id, tol)
 
   ## step3 : splitting the lines
   if(verbose){
     print("Building graph ...")
   }
-  new_lines <- simple_lines(new_lines)
+  #new_lines <- simple_lines(new_lines)
   new_lines$length <- gLength(new_lines,byid = TRUE)
   new_lines <- subset(new_lines,new_lines$length>0)
   new_lines <- remove_loop_lines(new_lines,digits)
